Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.Windows
Imports System.Windows.Documents

Namespace FlowDocumentApp.FlowDocuments
	Public Class ItemsContent
		Inherits Section
        Private Shared ReadOnly ItemsSourceProperty As DependencyProperty = DependencyProperty.Register("ItemsSource", GetType(IEnumerable), GetType(ItemsContent), New PropertyMetadata(AddressOf OnItemsSourceChanged))
        Private Shared ReadOnly ItemTemplateProperty As DependencyProperty = DependencyProperty.Register("ItemTemplate", GetType(DataTemplate), GetType(ItemsContent), New PropertyMetadata(AddressOf OnItemTemplateChanged))
        Private Shared ReadOnly ItemsPanelProperty As DependencyProperty = DependencyProperty.Register("ItemsPanel", GetType(DataTemplate), GetType(ItemsContent), New PropertyMetadata(AddressOf OnItemsPanelChanged))

		Public Sub New()
			Helpers.FixupDataContext(Me)
			AddHandler Loaded, AddressOf ItemsContent_Loaded
		End Sub

		Private Sub ItemsContent_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			GenerateContent(ItemsPanel, ItemTemplate, ItemsSource)
		End Sub

		Public Property ItemsSource() As IEnumerable
			Get
				Return CType(GetValue(ItemsSourceProperty), IEnumerable)
			End Get
			Set(ByVal value As IEnumerable)
				SetValue(ItemsSourceProperty, value)
			End Set
		End Property

		Public Property ItemTemplate() As DataTemplate
			Get
				Return CType(GetValue(ItemTemplateProperty), DataTemplate)
			End Get
			Set(ByVal value As DataTemplate)
				SetValue(ItemTemplateProperty, value)
			End Set
		End Property

		Public Property ItemsPanel() As DataTemplate
			Get
				Return CType(GetValue(ItemsPanelProperty), DataTemplate)
			End Get
			Set(ByVal value As DataTemplate)
				SetValue(ItemsPanelProperty, value)
			End Set
		End Property

		Private Sub GenerateContent(ByVal itemsPanel As DataTemplate, ByVal itemTemplate As DataTemplate, ByVal itemsSource As IEnumerable)
			Blocks.Clear()
			If itemTemplate IsNot Nothing AndAlso itemsSource IsNot Nothing Then
				Dim panel As FrameworkContentElement = Nothing

				For Each data As Object In itemsSource
					If panel Is Nothing Then
						If itemsPanel Is Nothing Then
							panel = Me
						Else
							Dim p As FrameworkContentElement = Helpers.LoadDataTemplate(itemsPanel)
							If Not(TypeOf p Is Block) Then
								Throw New Exception("ItemsPanel must be a block element")
							End If
							Blocks.Add(CType(p, Block))
							panel = Attached.GetItemsHost(p)
							If panel Is Nothing Then
								Throw New Exception("ItemsHost not found. Did you forget to specify Attached.IsItemsHost?")
							End If
						End If
					End If
					Dim element As FrameworkContentElement = Helpers.LoadDataTemplate(itemTemplate)
					element.DataContext = data
					Helpers.UnFixupDataContext(element)
					If TypeOf panel Is Section Then
						CType(panel, Section).Blocks.Add(Helpers.ConvertToBlock(data, element))
					ElseIf TypeOf panel Is TableRowGroup Then
						CType(panel, TableRowGroup).Rows.Add(CType(element, TableRow))
					Else
						Throw New Exception(String.Format("Don't know how to add an instance of {0} to an instance of {1}", element.GetType(), panel.GetType()))
					End If
				Next data
			End If
		End Sub

		Private Sub GenerateContent()
			GenerateContent(ItemsPanel, ItemTemplate, ItemsSource)
		End Sub

		Private Sub OnItemsSourceChanged(ByVal newValue As IEnumerable)
			If IsLoaded Then
				GenerateContent(ItemsPanel, ItemTemplate, newValue)
			End If
		End Sub

		Private Sub OnItemTemplateChanged(ByVal newValue As DataTemplate)
			If IsLoaded Then
				GenerateContent(ItemsPanel, newValue, ItemsSource)
			End If
		End Sub

		Private Sub OnItemsPanelChanged(ByVal newValue As DataTemplate)
			If IsLoaded Then
				GenerateContent(newValue, ItemTemplate, ItemsSource)
			End If
		End Sub

		Private Shared Sub OnItemsSourceChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
			CType(d, ItemsContent).OnItemsSourceChanged(CType(e.NewValue, IEnumerable))
		End Sub

		Private Shared Sub OnItemTemplateChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
			CType(d, ItemsContent).OnItemTemplateChanged(CType(e.NewValue, DataTemplate))
		End Sub

		Private Shared Sub OnItemsPanelChanged(ByVal d As DependencyObject, ByVal e As DependencyPropertyChangedEventArgs)
			CType(d, ItemsContent).OnItemsPanelChanged(CType(e.NewValue, DataTemplate))
		End Sub
	End Class
End Namespace
